# ResourcePlanner

[![pipeline status](https://gitlab.htcl.eu/nerebext/resource_planner/badges/master/pipeline.svg)](https://gitlab.htcl.eu/nerebext/resource_planner/commits/master)
[![coverage report](https://gitlab.htcl.eu/nerebext/resource_planner/badges/master/coverage.svg)](https://gitlab.htcl.eu/nerebext/resource_planner/commits/master)

Short description and motivation.

## Usage

How to use my plugin.

## Installation

Add this line to your application's Gemfile:

```ruby
gem 'resource_planner'
```

And then execute:

```bash
bundle
```

Or install it yourself as:

```bash
gem install resource_planner
```

## Setup Notes

```bash
rails plugin new resource_planner --mountable -d sqlite3 -T --dummy-path=spec/dummy --skip-bundle
```

Add the following to the gemspec file:

```ruby
  s.rdoc_options = ["--line-numbers", "--inline-source", "--title", "resource_planner", "--main", "README.rdoc"]
  s.extra_rdoc_files = ["CHANGELOG", "README.rdoc", "lib/task_util.rb", "lib/tasks/resource_planner.rake", "lib/resource_planner.rb"]
```

AND

```ruby
  s.add_development_dependency 'rspec-rails', '~> 2.8.1'
  s.add_development_dependency 'cucumber-rails', '~> 1.3.0'
```

```bash
rails g rspec:install
rails g cucumber:install --rspec --capybara
```

Add this to the engine file:

 ```ruby
   config.generators do |g|
      g.test_framework :rspec, :view_specs => false
    end
```

Add something like this to spec/spec_helper.rb

```ruby
require File.expand_path("../test/dummy/config/environments", __FILE__)
```

Create features/support/cucumber.rb , containing something like this:

```ruby
require File.expand_path(File.dirname(__FILE__) + '../../test/dummy/config/environments')
```

Generate the home page:

```bash
rails g controller Overview index
```

Add the following to config/routes.rb:

```ruby
  root :to => "overview#index"
```

## Model definitions

```bash
rails generate model Event name:string description:string start_at:datetime end_at:datetime all_day:boolean
rails generate model EventSubscription name:string event:references resource:references{polymorphic}
rails generate model AvailabiltyDateDefinition name:string resource:references{polymorphic} availabilty_date:references
rails generate model AvailabiltyRestrictionDefinition name:string resource:references{polymorphic} availabilty_restriction:references

rails g scaffold Event
rails g controller Resources list events

cd spec/dummy
rails generate model Course name:string type:string description:string
rails generate model Person name:string
rails generate model Room name:string
rails generate model Equipment name:string
```

## Database initialisation/Migration

```bash
rails app:db:environment:set RAILS_ENV=development && RAILS_ENV=development rake db:drop db:create db:migrate db:seed
rails app:db:environment:set RAILS_ENV=test && RAILS_ENV=test rake db:drop db:create db:migrate
```

## ICalendar Specification

[ICalendar (Wikipedia)](https://en.wikipedia.org/wiki/ICalendar)

[ICalendar Specification](https://icalendar.org/RFC-Specifications/iCalendar-RFC-5545)

[jCal: The JSON Format for iCalendar](https://tools.ietf.org/html/rfc7265)

![ICalendar Specification](doc/ICalendarSpecification.png)

### Example

Here is a simple example of an iCalendar object, "Bastille Day Party" event which occurs July 14, 1997 17:00 (UTC) through July 15, 1997 03:59:59 (UTC):

```text
BEGIN:VCALENDAR
VERSION:2.0
PRODID:-//hacksw/handcal//NONSGML v1.0//EN
BEGIN:VEVENT
UID:uid1@example.com
DTSTAMP:19970714T170000Z
ORGANIZER;CN=John Doe:MAILTO:john.doe@example.com
DTSTART:19970714T170000Z
DTEND:19970715T035959Z
SUMMARY:Bastille Day Party
GEO:48.85299;2.36885
END:VEVENT
END:VCALENDAR
```

## Contributing

Contribution directions go here.

## License

This gem is copyrighted

---

## git-svn notes

Import project from subversion repository:

```bash
git svn clone --trunk=trunk/resource_planner --tags=tags/resource_planner --branches=branches/resource_planner --authors-file=/home/everard/.git-authors https://dev.htcl.eu/project/repo/website/rails_engines resource_planner
```

New branch linked to subversion repository:

```bash
git svn branch -m "Create feature branch" <feature_branch>
git checkout --track -b <feature_branch> <feature_branch>
```

Integration:

```bash
git merge <feature_branch>
```

OR

```bash
git merge --squash <feature_branch>
```

Get latest changes from subversion repository:

```bash
git svn rebase
```

Commit to subversion repository:

```bash
git svn rebase && git svn dcommit
```

Synchronising branches:

Integrate 'wip' into 'development':

```bash
git stash save && git checkout wip && git rebase development && git checkout development && git merge wip && git checkout wip && git rebase development && git stash pop
```

Integrate 'development' into 'master':

```bash
git stash save && git checkout master && git svn rebase && git checkout development && git rebase master && git checkout master && git merge development && git svn dcommit && git checkout development && git rebase master && git stash pop
```

Re-base 'wip' from 'development':

```bash
git stash save && git checkout wip && git rebase development && git stash pop
```

Re-base 'development' from 'master':

```bash
git stash save && git checkout master && git svn rebase && git checkout development && git rebase master && git stash pop
```

Sync 'master' to subversion

```bash
git stash save && git checkout master && git svn dcommit && git stash pop
```

Sync to all shared repositories:

```bash
git gc && for branch in master development ; do [[ `git rev-parse --verify --quiet $branch` ]] && (git stash save 'repo-sync' && git pull --rebase gitlab $branch && ([[ `git stash list | grep 'repo-sync'` ]] && git stash pop) ; (for repo in origin tfs gitlab ; do git push -u $repo $branch ; done)) ; done
```
