$(document).ready(function () {
  /******************************************** EVENT HANDLERS ***********************************************/
  /******************************************* Timezone Handlers *********************************************/
  $('#resource_planner_event_dtstart_timezone').on('change', function () {
    renderHighlighterBar('timezone');
  });

  $('#resource_planner_event_dtstart_timezone').on('input', function () {
    renderHighlighterBar('timezone');
  });
});

//__________________________________________________________________________________
// Return highlighter object
//==================================================================================
function getHighlighterBar() {
  return $('#highlighter');
}

//__________________________________________________________________________________
// Calculates the UTC Offsets
//==================================================================================
function tzOffset() {                                                 // Calculates the UTC Offsets
  var startTZ = $("#resource_planner_event_dtstart_timezone").val();  // Read the timezone selected
  var startZone = moment;
  startZone.tz.setDefault(startTZ)();                                 // Set the Timezone
  var utcOffset = startZone().utcOffset() * 60000;                    // Total UTC offset in milliseconds

  return utcOffset;
}

//================================================================================================
function getEventDurationHours() {
  var view = fullcalendarScheduler.view;

  var viewDuration = view.currentEnd.diff(view.currentStart, "hours");
  var eventDurationHours = viewDuration * getHighlighterBar().width() / getEventWindowWidth();

  return eventDurationHours;
}

//================================================================================================
function setHighlighterBarWidth(duration, durationUnits) {
  var view = fullcalendarScheduler.view;

  var eventDurationHours;
  switch (durationUnits) {
    case 'day':
      eventDurationHours = duration * 24.0;
      break;
    case 'hour':
      eventDurationHours = duration;
      break;
    case 'minute':
      eventDurationHours = duration / 60.0;
      break;
    default:
      eventDurationHours = duration / 60.0;
  }

  var viewDuration = view.currentEnd.diff(view.currentStart, "hours");
  var width = getEventWindowWidth() * eventDurationHours / viewDuration;

  getHighlighterBar().css('width', width);

  return width;
}

//================================================================================================
function renderDurationField() {
  var eventDurationHours = getEventDurationHours();
  var durationUnits = $.trim($(formField().DURATION_UNITS).val());

  var duration;
  switch (durationUnits) {
    case 'day':
      duration = (eventDurationHours / 24).toFixed(2);
      break;
    case 'hour':
      duration = (eventDurationHours).toFixed(2);
      break;
    case 'minute':
      duration = Math.round(eventDurationHours * 60);
      break;
    default:
      duration = Math.round(eventDurationHours * 60);
  }

  $(formField().DURATION).val(duration);
  $("#resource_planner_event_duration").val(duration);
}

//================================================================================================
function renderEndDateField() {
  var startDate = new Date($.trim($(formField().DTSTART).val()));
  var duration = $.trim($(formField().DURATION).val());
  var durationUnits = $.trim($(formField().DURATION_UNITS).val());

  var endDate = dateAdd(startDate, duration, durationUnits);

  if (isNaN(endDate) && isNaN(startDate)) {
    duration = "";
    $(formField().DURATION).val(duration);
  } else {
    $(formField().DTEND).val(formatDate(endDate));
    $(formField().DURATION).val(duration);
  }
}

//================================================================================================
function onHighlighterBarUpdate(event, ui, obj) {     //Function is triggered by either DRAG or RESIZE events
  var isRTL = (fullcalendarScheduler.getOption('dir') == 'rtl');
  var view = fullcalendarScheduler.view;
  var viewStartDate = new Date(view.currentStart);

  var eventStartTimeHours;
  var eventEndTimeHours;
  var viewDuration = view.currentEnd.diff(view.currentStart, "hours");
  var eventDurationHours = viewDuration * obj.width() / getEventWindowWidth();

  if (isRTL) {
    eventEndTimeHours = getEventWindowHours() - (getEventWindowHours() * ui.position.left / getEventWindowWidth());
    eventStartTimeHours = eventEndTimeHours - eventDurationHours;
  } else {
    eventStartTimeHours = getEventWindowHours() * ui.position.left / getEventWindowWidth();
  }

  // Date object methods
  // setDate()	Sets the day of the month of a date object
  // setFullYear()	Sets the year of a date object
  // setHours()	Sets the hour of a date object
  // setMilliseconds()	Sets the milliseconds of a date object
  // setMinutes()	Set the minutes of a date object
  // setMonth()	Sets the month of a date object

  var startDate = dateAdd(viewStartDate, parseInt(eventStartTimeHours / 24), 'day');
  startDate.setHours(Math.floor(eventStartTimeHours % 24));
  startDate.setMinutes(60 * (eventStartTimeHours % 1).toFixed(10));

  var id = obj.attr('id');
  if (id == 'highlighter') {
    // The old highlighter bar
    //$(formField().DTSTART).val(formatDate(startDate));
    //renderDurationField();
    //renderEndDateField();
  } else {
    var eventIndex = id.split('-')[1];
    var duration = getDurationInUnits(eventDurationHours, window.eventsArray[eventIndex].durationUnits);
    window.eventsArray[eventIndex].startDate = moment(startDate).format();
    window.eventsArray[eventIndex].eventDuration = duration.toString();
    window.eventsArray[eventIndex].endDate = dateAdd(window.eventsArray[eventIndex].startDate, window.eventsArray[eventIndex].eventDuration, window.eventsArray[eventIndex].durationUnits);
    window.eventsArray[eventIndex].isDirty = true;

    if (noReccurence) {
      $.trim($(formField().DURATION).val(duration));
      var end_date = dateAdd(startDate, duration, window.eventsArray[eventIndex].durationUnits);
      $(formField().DTEND).val(formatDate(end_date));
      $(formField().DTSTART).val(formatDate(startDate));
    }
  }
}

//================================================================================================
function renderHighlighterBar(trigger) {  //function triggered when '.date-field' or formField().DURATION fields changed

  if (getHighlighterBar().length == 0) {
    $('#fullcalendar_scheduler tbody.fc-body td.fc-time-area div.fc-scroller div.fc-scroller-canvas').prepend($('<div id="highlighter" draggable="true"></div>'));
  }

  tzOffset();

  getHighlighterBar().on('drag', function (event, ui) {
    onHighlighterBarUpdate(event, ui, $(this));
  });

  getHighlighterBar().on('resize', function (event, ui) {
    onHighlighterBarUpdate(event, ui, $(this));
  });

  var start_date = $.trim($(formField().DTSTART).val());
  var end_date = $.trim($(formField().DTEND).val());
  var duration = $.trim($(formField().DURATION).val());
  var durationUnits = $.trim($(formField().DURATION_UNITS).val());

  var isRTL = (fullcalendarScheduler.getOption('dir') == 'rtl');
  var view = fullcalendarScheduler.view;

  var viewStartDate = new Date(view.currentStart);
  var viewEndDate = new Date(view.currentEnd);

  var viewStartTime = viewStartDate.getTime();
  var viewEndTime = viewEndDate.getTime();
  var viewDuration = viewEndTime - viewStartTime;

  if ((start_date.length && end_date.length) || (duration.length) || (start_date == "")) {
    var startDate = new Date(start_date);
    var endDate = new Date(end_date);

    var width;
    // End Date must be >= start Date
    if ((endDate.getTime() <= startDate.getTime()) || isNaN(endDate) || isNaN(startDate)) {
      end_date = $(formField().DTSTART).val();
      endDate = new Date(end_date);
      $(formField().DTEND).val(end_date);
      duration = "";
      $(formField().DURATION).val(duration);
      width = 0;
      getHighlighterBar().css('width', width)
    }

    switch (trigger) {
      case 'duration':
        if (duration.length) {
          width = setHighlighterBarWidth(duration, durationUnits)

          if (isRTL) {
            getHighlighterBar().css('left', getEventWindowWidth() - ((((startDate.getTime() + tzOffset()) - viewStartTime) / viewDuration) * getEventWindowWidth()) - width);
          } else {
            getHighlighterBar().css('left', (((startDate.getTime() - viewStartTime) / viewDuration) * view.timeAxisTbody.clientWidth));
            //getHighlighterBar().css('left', (((startDate.getTime() + tzOffset()) - viewStartTime) / viewDuration) * view.timeBodyScroller.canvas.width);
          }
        }
        break;

      case 'date':
      default:
        if (end_date.length) {
          if (isRTL) {
            getHighlighterBar().css('left', (view.timeAxisTbody.clientWidth - ((((endDate.getTime() - viewStartDate.getTime()) + tzOffset()) / viewDuration) * view.timeAxisTbody.clientWidth)));
            //getHighlighterBar().css('left', (view.timeBodyScroller.canvas.width - ((((endDate.getTime() - viewStartDate.getTime()) + tzOffset()) / viewDuration) * view.timeBodyScroller.canvas.width)));
          } else {
            getHighlighterBar().css('left', (((startDate.getTime() - viewStartTime) / viewDuration) * view.timeAxisTbody.clientWidth));
            //getHighlighterBar().css('left', (((startDate.getTime() + tzOffset()) - viewStartTime) / viewDuration) * view.timeBodyScroller.canvas.width);
          }
          setHighlighterBarWidth(endDate.diff(startDate, durationUnits), durationUnits);
        }
        break;
    }

    getHighlighterBar().height(getEventWindowHeight());
    getHighlighterBar().draggable({ axis: "x" });
    getHighlighterBar().resizable({ handles: "e, w" });
    getHighlighterBar().css('visibility', 'visible'); // TODO: Move this inside the if block

    if ((getHighlighterBar()[0].clientWidth > 0 && (endDate.getTime() > startDate.getTime())) || (getHighlighterBar()[0].clientWidth > 0 && isNaN(endDate.getTime()))) {
      renderDurationField();
    }
  }
}

//__________________________________________________________________________________
// Function to prepare events for rendering
//==================================================================================
function prepareEvents(recurenceObject, recurencePattern) {
  var end_date;
  var start_date;
  var selectedWeekDayNumber;
  var startWeekDayNumber;
  var i;
  var now;
  var end;
  var duration;
  var eventsArray = [];
  var nrOfOccurences;
  var found = false;

  ViewstartDate = recurenceObject.startDate;
  ViewendDate = recurenceObject.endDate;

  switch (recurencePattern) {                                                          // SWITCH FOR SINGLE EVENT, DAILY, WEEKLY or MONTHLY
    case "daily":               //               ***** D A I L Y *****
      if (recurenceObject.nrOfEvents) {
        for (i = 0; i < recurenceObject.nrOfEvents; i++) {                             // number of selected days is greater than number of events
          eventsArray.push({});
        }

        for (i = 0; i < recurenceObject.nrOfEvents; i++) {
          if (i == 0) {
            eventsArray[i].startDate = recurenceObject.startDate;
            eventsArray[i].eventDuration = recurenceObject.eventDuration;
            eventsArray[i].durationUnits = recurenceObject.durationUnits;
            ViewendDate = eventsArray[i].startDate;
          } else {
            start_date = getDailyEventStartDate(eventsArray[i - 1].startDate, recurenceObject.howOften);
            eventsArray[i].startDate = start_date;
            eventsArray[i].eventDuration = recurenceObject.eventDuration;
            eventsArray[i].durationUnits = recurenceObject.durationUnits;
            ViewendDate = eventsArray[i].startDate;
          }
        }

      } else {
        now = moment(recurenceObject.startDate);
        end = moment(recurenceObject.endDate);
        nrOfOccurences = Math.round((moment(end).diff(now, 'days') + 2) / recurenceObject.howOften);

        for (i = 0; i < nrOfOccurences; i++) {                             // number of selected days is greater than number of events
          eventsArray.push({});
        }

        for (i = 0; i < nrOfOccurences; i++) {
          if (i == 0) {
            eventsArray[i].startDate = recurenceObject.startDate;
            eventsArray[i].eventDuration = recurenceObject.eventDuration;
            eventsArray[i].durationUnits = recurenceObject.durationUnits;
          } else {
            start_date = getDailyEventStartDate(eventsArray[i - 1].startDate, recurenceObject.howOften);
            eventsArray[i].startDate = start_date;
            eventsArray[i].eventDuration = recurenceObject.eventDuration;
            eventsArray[i].durationUnits = recurenceObject.durationUnits;
          }
        }
      }

      ViewstartDate = eventsArray[0].startDate;
      ViewendDate = eventsArray[eventsArray.length - 1].startDate;

      for (var i = 0; i < eventsArray.length; i++) {
        eventsArray[i].endDate = dateAdd(eventsArray[i].startDate, eventsArray[i].eventDuration, eventsArray[i].durationUnits);
        eventsArray[i].initialStartDate = eventsArray[i].startDate;
        eventsArray[i].initialEventDuration = eventsArray[i].eventDuration;
        eventsArray[i].initialDurationUnits = eventsArray[i].durationUnits;
        eventsArray[i].isDirty = false;
      }

      window.eventsArray = eventsArray;
      window.StartingDate = ViewstartDate;
      window.endingDate = ViewendDate;
      window.flag = true;
      window.zoomed = false;

      updateTimelineDuration(ViewstartDate, ViewendDate);
      //renderEvents(eventsArray, ViewstartDate, ViewendDate);

      break;

    case "weekly":    //               ***** W  E  E  K  L  Y *****
      var weekDaysNumberArray = [];
      var selectedDayOfWeekNumber;
      var numberOfDays;
      var StartDayOfWeek = moment(recurenceObject.startDate);
      var StartDayOfWeekNumber = StartDayOfWeek.day();

      for (i = 0; i < recurenceObject.when.length; i++) {     // create a new array to hold week days number
        weekDaysNumberArray.push({});
      }

      for (i = 0; i < recurenceObject.when.length; i++) {     // update the array with the selectedweekdays to hold day numbers
        selectedDayOfWeekNumber = moment().day(recurenceObject.when[i]);
        weekDaysNumberArray[i] = selectedDayOfWeekNumber.day();
      }

      for (i = 0; i < weekDaysNumberArray.length; i++) {      // find the start day dependent on the startDate (selected in the menu)
        if (StartDayOfWeekNumber <= weekDaysNumberArray[i]) {
          numberOfDays = weekDaysNumberArray[i] - StartDayOfWeekNumber;
          start_date = moment(recurenceObject.startDate).add(numberOfDays, 'days');
          var index = i;                                      // index of the found next day...
          found = true;
          break;
        }
      }

      if (!found) {
        for (i = 0; i < weekDaysNumberArray.length; i++) {
          if (StartDayOfWeekNumber >= weekDaysNumberArray[i]) {
            numberOfDays = (7 - StartDayOfWeekNumber) + 1;
            start_date = moment(recurenceObject.startDate).add(numberOfDays, 'days');
            break;
          }
        }
      }

      var partOfArray = [];
      if (found) {
        partOfArray = weekDaysNumberArray.slice(index);
        for (i = 0; i < partOfArray.length; i++) {
          weekDaysNumberArray.pop();
        }
        weekDaysNumberArray = partOfArray.concat(weekDaysNumberArray);
      }

      var StartDayOfWeek = moment(start_date);

      if (recurenceObject.nrOfEvents == "") {
        var totalDayOfAppearance = 0;
        var dayNumberOfAppearances = 0;
        for (i = 0; i < weekDaysNumberArray.length; i++) {
          dayNumberOfAppearances = countSelectedWeekdayAppearance(start_date, recurenceObject.endDate, weekDaysNumberArray[i]);
          dayNumberOfAppearances = Math.ceil(dayNumberOfAppearances / recurenceObject.howOften);
          totalDayOfAppearance = totalDayOfAppearance + dayNumberOfAppearances;
          dayNumberOfAppearances = 0;
        }

        for (i = 0; i < totalDayOfAppearance; i++) {                   // number of selected days is greater than number of events
          eventsArray.push({});
        }
      } else {
        for (i = 0; i < recurenceObject.nrOfEvents; i++) {                   // number of selected days is greater than number of events
          eventsArray.push({});
        }
      }

      if (recurenceObject.nrOfEvents) {    // Number of events greater than zero                                                          // number of events is selected

        // calculate the end_date
        var nrOfWeeks = (recurenceObject.nrOfEvents * recurenceObject.howOften) + recurenceObject.howOften; // number of weeks until the end date
        end_date = moment(start_date);
        var ViewendDate = moment(end_date).add(nrOfWeeks, 'weeks');
        var ViewstartDate = moment(start_date);


        var j;
        for (i = 0; i < recurenceObject.nrOfEvents; i++) {
          startWeekDayNumber = moment(start_date).day();         // get the weekday number of the start date, e.g. sunday=0, monday=1,... here we get the startDate day number
          var index = i % weekDaysNumberArray.length;

          if (index == 0 && i != 0) {
            numberOfDays = 7 * recurenceObject.howOften;
            start_date = moment(start_date).add(numberOfDays, 'days');
          }

          eventsArray[i].startDate = getWeeklyEventStartDate(weekDaysNumberArray[index], startWeekDayNumber, start_date);
          eventsArray[i].eventDuration = recurenceObject.eventDuration;
          eventsArray[i].durationUnits = recurenceObject.durationUnits;
        }

        ViewstartDate = eventsArray[0].startDate;
        ViewendDate = eventsArray[eventsArray.length - 1].startDate;

        for (var i = 0; i < eventsArray.length; i++) {
          eventsArray[i].endDate = dateAdd(eventsArray[i].startDate, eventsArray[i].eventDuration, eventsArray[i].durationUnits);
          eventsArray[i].initialStartDate = eventsArray[i].startDate;
          eventsArray[i].initialEventDuration = eventsArray[i].eventDuration;
          eventsArray[i].initialDurationUnits = eventsArray[i].durationUnits;
          eventsArray[i].isDirty = false;
        }

        window.eventsArray = eventsArray;
        window.StartingDate = ViewstartDate;
        window.endingDate = ViewendDate;
        window.flag = true;
        window.zoomed = false;

        updateTimelineDuration(ViewstartDate, ViewendDate);
        //ents(eventsArray, ViewstartDate, ViewendDate);

        break;

      } else {
        ViewendDate = recurenceObject.endDate;
        ViewstartDate = recurenceObject.startDate;

        var arrayStart = 0;
        var count = 0;
        for (i = 0; i < weekDaysNumberArray.length; i++) {
          startWeekDayNumber = moment(start_date).day();
          selectedWeekDayNumber = moment().day(weekDaysNumberArray[i]);                      // get the weekday name from the array (e.g. 'Monday', 'Thursday',...)
          selectedWeekDayNumber = selectedWeekDayNumber.day();
          dayNumberOfAppearances = countSelectedWeekdayAppearance(start_date, recurenceObject.endDate, weekDaysNumberArray[i]);
          var startDate = getWeeklyEventStartDate(selectedWeekDayNumber, startWeekDayNumber, start_date);

          count = count + Math.ceil(dayNumberOfAppearances / recurenceObject.howOften);
          for (j = arrayStart; j < count; j++) {
            eventsArray[j].startDate = moment(startDate).format();
            eventsArray[j].eventDuration = recurenceObject.eventDuration;
            eventsArray[j].durationUnits = recurenceObject.durationUnits;
            startDate = moment(startDate).add((recurenceObject.howOften * 7), 'days');
          }
          arrayStart = j;
        }

        eventsArray.sort(sortArray);
      }

      ViewstartDate = eventsArray[0].startDate;
      ViewendDate = eventsArray[eventsArray.length - 1].startDate;

      for (var i = 0; i < eventsArray.length; i++) {
        eventsArray[i].endDate = dateAdd(eventsArray[i].startDate, eventsArray[i].eventDuration, eventsArray[i].durationUnits);
        eventsArray[i].initialStartDate = eventsArray[i].startDate;
        eventsArray[i].initialEventDuration = eventsArray[i].eventDuration;
        eventsArray[i].initialDurationUnits = eventsArray[i].durationUnits;
        eventsArray[i].isDirty = false;
      }

      window.eventsArray = eventsArray;
      window.StartingDate = ViewstartDate;
      window.endingDate = ViewendDate;
      window.flag = true;
      window.zoomed = false;

      updateTimelineDuration(ViewstartDate, ViewendDate);
      //renderEvents(eventsArray, ViewstartDate, ViewendDate);

      break;

    case "monthly":    //              ***** M O N T H L Y *****
      weekDaysNumberArray = [];
      var startDateOfSelectedDayOfMonth;
      var selectedDayWeekNumber;
      var weekNumber;
      var order;

      //===========================================================================================================================
      if (isNaN(recurenceObject.when[0])) {      // "recurence_pattern" seelcted is e.g. "Every", "The first", "The second",... was selected

        var selectedDayWeekDate = moment().day(recurenceObject.when[1]);
        var selectedDayWeekNumber = moment(selectedDayWeekDate).day();
        var startDateOfMonth = moment(recurenceObject.startDate).startOf('month');
        var startDateOfMonthWeekNumber = startDateOfMonth.day();
        order = getOrder(recurenceObject);

        //_____________________________________________________________________________________________________________
        if (recurenceObject.nrOfEvents) {                              // "create X events" of range_of_recurence was selected (for NaN)

          for (i = 0; i < recurenceObject.nrOfEvents; i++) {           // number of selected days is greater than number of events
            eventsArray.push({});
          }

          if (order < 5) {                                             // one of "every", "first", "second", "third", "fourth" was selected
            start_date = getStartDateForSelectedWeekday(recurenceObject);
            selectedDayNumber = moment(start_date).day();
            start_date = new Date(start_date);
            var start_month = start_date.getMonth();

            for (i = 0; i < recurenceObject.nrOfEvents; i++) {          // get all events for the selected day of the week
              start_date = new Date(start_date);
              if (start_month == start_date.getMonth()) {
                eventsArray[i].startDate = moment(start_date).format();
                eventsArray[i].eventDuration = recurenceObject.eventDuration;
                eventsArray[i].durationUnits = recurenceObject.durationUnits;
                if (order == 0) {
                  start_date = moment(start_date).add(1, 'weeks');
                } else {
                  start_date = moment(start_date).add(recurenceObject.howOften, 'months');
                  var startDateOfMonth = moment(start_date).startOf('month');
                  var startDateOfMonthWeekNumber = startDateOfMonth.day();
                  if (i == 0) {
                    start_date = getTheNextEventDate(eventsArray[i].startDate, order, recurenceObject);
                  } else {
                    start_date = getTheNextEventDate(eventsArray[i - 1].startDate, order, recurenceObject);
                  }
                }

              } else {
                start_date = moment(eventsArray[i - 1].startDate).add(recurenceObject.howOften, 'months');
                var startDateOfMonth = moment(start_date).startOf('month');
                var startDateOfMonthWeekNumber = startDateOfMonth.day();
                if (i == 0) {
                  start_date = getTheNextEventDate(eventsArray[i].startDate, order, recurenceObject);
                } else {
                  start_date = getTheNextEventDate(eventsArray[i - 1].startDate, order, recurenceObject);
                }
                eventsArray[i].startDate = moment(start_date).format();
                eventsArray[i].eventDuration = recurenceObject.eventDuration;
                eventsArray[i].durationUnits = recurenceObject.durationUnits;
                start_date = new Date(start_date);
                start_month = start_date.getMonth();
                if (order == 0) {
                  start_date = moment(start_date).add(1, 'weeks');
                } else {
                  start_date = moment(start_date).add(recurenceObject.howOften, 'months');
                }
              }
            }                                             // End of - get all events for the selected day of the week
          }                                               // End of - one of "every", "first", "second", "third", "fourth" was selected

          if (order >= 5) {                               // Either "fifth" or "last" were selected
            selectedDayWeekDate = moment().day(recurenceObject.when[1]);
            var selectedDayNumber = moment(selectedDayWeekDate).day();
            if (order == 5) {                             // Option "fifth" was selected
              var valid = true;
              for (i = 0; i < recurenceObject.nrOfEvents; i++) {
                valid = true;
                while (valid) {
                  start_date = new Date(startDateOfMonth);
                  var startYear = start_date.getFullYear();
                  var startMonth = start_date.getMonth();
                  var NumberOfDaysInMonth = moment(startDateOfMonth).daysInMonth();
                  if (NumberOfDaysInMonth > 28) {
                    for (j = 29; j <= NumberOfDaysInMonth; j++) {
                      startDateOfMonth = moment([startYear, startMonth, j]);
                      startDateOfMonth = new Date(startDateOfMonth);
                      var startingDate = new Date(recurenceObject.startDate);
                      var startDateNumber = moment(startDateOfMonth).day();
                      if ((startDateNumber == selectedDayNumber) && (moment(startDateOfMonth) >= moment(startingDate))) {
                        eventsArray[i].startDate = moment(startDateOfMonth).format();
                        eventsArray[i].eventDuration = recurenceObject.eventDuration;
                        eventsArray[i].durationUnits = recurenceObject.durationUnits;
                        valid = false;
                      }
                    }
                  }
                  startDateOfMonth = moment(startDateOfMonth).startOf('month');
                  startDateOfMonth = moment(startDateOfMonth).add((1 * recurenceObject.howOften), 'months');
                }
              }
            }                                       // End of - Option "fifth" was selected

            if (order == 6) {                      // Option "last" was selected
              var startDay = moment(recurenceObject.startDate).date();
              var limitDay = moment(recurenceObject.startDate).daysInMonth();
              start_date = new Date(recurenceObject.startDate);
              startYear = start_date.getFullYear();
              startMonth = start_date.getMonth();
              var j = 0;
              for (i = startDay; i <= limitDay; i++) {
                var newDate = moment([startYear, startMonth, i]);
                var newDateNumber = moment(newDate).day();

                if (newDateNumber == selectedDayNumber) {
                  eventsArray[j].startDate = moment(newDate).format();
                  eventsArray[j].eventDuration = recurenceObject.eventDuration;
                  eventsArray[j].durationUnits = recurenceObject.durationUnits;
                  j++;

                }
              }

              for (i = j; i < recurenceObject.nrOfEvents; i++) {
                startDateOfMonth = moment(startDateOfMonth).startOf('month');
                startDateOfMonth = moment(startDateOfMonth).add((1 * recurenceObject.howOften), 'months');
                NumberOfDaysInMonth = moment(startDateOfMonth).daysInMonth();
                start_date = new Date(startDateOfMonth);
                startYear = start_date.getFullYear();
                startMonth = start_date.getMonth();
                valid = true;

                while (valid) {
                  newDate = moment([startYear, startMonth, NumberOfDaysInMonth]);
                  newDateNumber = moment(newDate).day();
                  if (newDateNumber == selectedDayNumber) {
                    eventsArray[i].startDate = moment(newDate).format();
                    eventsArray[i].eventDuration = recurenceObject.eventDuration;
                    eventsArray[i].durationUnits = recurenceObject.durationUnits;
                    valid = false;
                  } else {
                    NumberOfDaysInMonth--;
                  }
                }
              }
            }                                     // End of - Option "last" was selected
          }                                       // End of - Either "fifth" or "last" were selected
          //}

          ViewstartDate = moment(eventsArray[0].startDate);
          ViewendDate = moment(eventsArray[eventsArray.length - 1].startDate);

          for (i = 0; i < eventsArray.length; i++) {
            eventsArray[i].endDate = dateAdd(eventsArray[i].startDate, eventsArray[i].eventDuration, eventsArray[i].durationUnits);
            eventsArray[i].initialStartDate = eventsArray[i].startDate;
            eventsArray[i].initialEventDuration = eventsArray[i].eventDuration;
            eventsArray[i].initialDurationUnits = eventsArray[i].durationUnits;
            eventsArray[i].isDirty = false;
          }

          window.eventsArray = eventsArray;

          window.StartingDate = ViewstartDate;
          window.endingDate = ViewendDate;
          window.flag = true;
          window.zoomed = false;

          updateTimelineDuration(ViewstartDate, ViewendDate);
          //renderEvents(eventsArray, ViewstartDate, ViewendDate);
        }                                                       // End of - "create X events" of range_of_recurence was selected (for NaN)

        //_____________________________________________________________________________________________________________
        if (recurenceObject.nrOfEvents == "") {                  // A "Repeat until" or "no_end_date" has bin selected (for NaN)

          if (order < 5) {
            start_date = getStartDateForSelectedWeekday(recurenceObject);
            start_date = new Date(start_date);
            start_month = start_date.getMonth();
            var selectedDayNumber = moment(start_date).day();
            index = 0;

            var begin_date = moment(start_date).startOf('day');
            while (moment(begin_date) <= moment(ViewendDate)) {
              if (begin_date > ViewendDate) {
                break;
              }
              start_date = new Date(start_date);
              if (start_month == start_date.getMonth()) {
                eventsArray.push({});
                eventsArray[index].startDate = moment(start_date).format();
                eventsArray[index].eventDuration = recurenceObject.eventDuration;
                eventsArray[index].durationUnits = recurenceObject.durationUnits;
                index++;
                if (order == 0) {
                  start_date = moment(start_date).add(1, 'weeks');
                } else {
                  start_date = moment(start_date).add(recurenceObject.howOften, 'months');
                  var startDateOfMonth = moment(start_date).startOf('month');
                  var startDateOfMonthWeekNumber = startDateOfMonth.day();
                  if (index == 0) {
                    start_date = getTheNextEventDate(eventsArray[index].startDate, order, recurenceObject);
                  } else {
                    start_date = getTheNextEventDate(eventsArray[index - 1].startDate, order, recurenceObject);
                  }
                }
              } else {
                if (index == 0) {
                  start_date = moment(eventsArray[index].startDate).add(recurenceObject.howOften, 'months');
                } else {
                  start_date = moment(eventsArray[index - 1].startDate).add(recurenceObject.howOften, 'months');
                }
                var startDateOfMonth = moment(start_date).startOf('month');
                var startDateOfMonthWeekNumber = startDateOfMonth.day();
                if (index == 0) {
                  start_date = getTheNextEventDate(eventsArray[index].startDate, order, recurenceObject);
                } else {
                  start_date = getTheNextEventDate(eventsArray[index - 1].startDate, order, recurenceObject);
                }
                if (start_date > ViewendDate) {
                  break;
                }
                eventsArray.push({});
                eventsArray[index].startDate = moment(start_date).format();
                eventsArray[index].eventDuration = recurenceObject.eventDuration;
                eventsArray[index].durationUnits = recurenceObject.durationUnits;
                start_date = new Date(start_date);
                start_month = start_date.getMonth();
                if (order == 0) {
                  start_date = moment(start_date).add(1, 'weeks');
                } else {
                  start_date = moment(start_date).add(recurenceObject.howOften, 'months');
                }
                index++;
              }
            }
          }

          if (order >= 5) {                                                // Selected order is either "fifth" or "last"
            selectedDayWeekDate = moment().day(recurenceObject.when[1]);
            selectedDayNumber = moment(selectedDayWeekDate).day();
            if (order == 5) {                                              // Begin of selected "fifth"
              i = 0;
              while (moment(startDateOfMonth) < moment(recurenceObject.endDate)) {
                start_date = new Date(startDateOfMonth);
                var startYear = start_date.getFullYear();
                var startMonth = start_date.getMonth();
                var NumberOfDaysInMonth = moment(startDateOfMonth).daysInMonth();
                if (NumberOfDaysInMonth > 28) {
                  for (j = 29; j <= NumberOfDaysInMonth; j++) {
                    startDateOfMonth = moment([startYear, startMonth, j]);
                    startDateOfMonth = new Date(startDateOfMonth);
                    var startingDate = new Date(recurenceObject.startDate);
                    var startDateNumber = moment(startDateOfMonth).day();
                    if ((startDateNumber == selectedDayNumber) && (moment(startDateOfMonth) >= moment(startingDate))) {
                      eventsArray.push({});
                      eventsArray[i].startDate = moment(startDateOfMonth).format();
                      eventsArray[i].eventDuration = recurenceObject.eventDuration;
                      eventsArray[i].durationUnits = recurenceObject.durationUnits;
                      i++;
                    }
                  }
                }
                startDateOfMonth = moment(startDateOfMonth).startOf('month');
                startDateOfMonth = moment(startDateOfMonth).add((1 * recurenceObject.howOften), 'months');
              }
            }                                                                           // End of selected "fifth"

            if (order == 6) {                                                           // Start of selected "last"
              selectedDayWeekDate = moment().day(recurenceObject.when[1]);
              selectedDayNumber = moment(selectedDayWeekDate).day();
              startDay = moment(recurenceObject.startDate).date();
              i = 0;

              while (moment(startDateOfMonth) <= moment(recurenceObject.endDate)) {     // Begin of WHILE
                start_date = new Date(startDateOfMonth);
                var startYear = start_date.getFullYear();
                var startMonth = start_date.getMonth();
                NumberOfDaysInMonth = moment(startDateOfMonth).daysInMonth();

                for (j = NumberOfDaysInMonth; j >= 20; j--) {
                  startDateOfMonth = moment([startYear, startMonth, j]);
                  startDateOfMonth = new Date(startDateOfMonth);
                  var startingDate = new Date(recurenceObject.startDate);
                  var startDateNumber = moment(startDateOfMonth).day();
                  if ((startDateNumber == selectedDayNumber) && (moment(startDateOfMonth) >= moment(startingDate))) {
                    eventsArray.push({});
                    eventsArray[i].startDate = moment(startDateOfMonth).format();
                    eventsArray[i].eventDuration = recurenceObject.eventDuration;
                    eventsArray[i].durationUnits = recurenceObject.durationUnits;
                    i++;
                    break;
                  }
                }
                startDateOfMonth = moment(startDateOfMonth).startOf('month');
                startDateOfMonth = moment(startDateOfMonth).add((1 * recurenceObject.howOften), 'months');
              }                                                               // End of WHILE
            }                                                                 // End of selected "last"
          }                                                                   // End of - Selected order is either "fifth" or "last"


          if (eventsArray.length > 0) {
            ViewstartDate = moment(eventsArray[0].startDate);
            ViewendDate = moment(eventsArray[eventsArray.length - 1].startDate);

            for (var i = 0; i < eventsArray.length; i++) {
              eventsArray[i].endDate = dateAdd(eventsArray[i].startDate, eventsArray[i].eventDuration, eventsArray[i].durationUnits);
              eventsArray[i].initialStartDate = eventsArray[i].startDate;
              eventsArray[i].initialEventDuration = eventsArray[i].eventDuration;
              eventsArray[i].initialDurationUnits = eventsArray[i].durationUnits;
              eventsArray[i].isDirty = false;
            }

            window.eventsArray = eventsArray;
            window.StartingDate = ViewstartDate;
            window.endingDate = ViewendDate;
            window.flag = true;
            window.zoomed = false;

            updateTimelineDuration(ViewstartDate, ViewendDate);
            //renderEvents(eventsArray, ViewstartDate, ViewendDate);

          } else {
            var untilDate = $('.range_of_recurence input#repeat_until.form-control.date-field.datepick');
            untilDate[0].value = "";
            $(".range_of_recurence").addClass("input_warning");
          }                                                        // End of - A "Repeat until" or "no_end_date" has bin selected (for NaN)
        }
      }                                                             // End of - "recurence_pattern" seelcted is e.g. "Every", "The first", "The second",... was selected

      //===========================================================================================================================
      if (!(isNaN(recurenceObject.when[0]))) {                     // "recurence_pattern" seelcted is e.g. 01, 02, 29,...,31

        if (recurenceObject.nrOfEvents) {                          // "create X events" of range_of_recurence was selected (for !isNaN)
          var arrayOfDays = [];
          for (i = 0; i < recurenceObject.nrOfEvents; i++) {       // number of selected days is greater than number of events
            eventsArray.push({});
          }

          for (i = 0; i < recurenceObject.when.length; i++) {      // find the start day dependent on the startDate (selected in the menu)
            if (moment(recurenceObject.startDate).date() <= recurenceObject.when[i]) {
              numberOfDays = recurenceObject.when[i] - moment(recurenceObject.startDate).date();
              start_date = moment(recurenceObject.startDate).add(numberOfDays, 'days');
              index = i;                                           // index of the found next day...
              found = true;
              break;
            }
          }

          if (!found) {
            var numberOfDaysInMonth = moment(recurenceObject.startDate).daysInMonth();
            numberOfDays = (numberOfDaysInMonth - recurenceObject.when[1]) + 1;
            start_date = moment(recurenceObject.startDate).add(numberOfDays, 'days');
            for (i = 0; i < recurenceObject.when.length; i++) {
              arrayOfDays.push({});
              arrayOfDays[i] = recurenceObject.when[i];
            }
          }

          if (found) {
            for (i = 0; i < recurenceObject.when.length; i++) {
              arrayOfDays.push({});
              arrayOfDays[i] = recurenceObject.when[i];
            }

            partOfArray = arrayOfDays.slice(index);
            for (j = 0; j < partOfArray.length; j++) {
              arrayOfDays.pop();
            }
            arrayOfDays = partOfArray.concat(arrayOfDays);
          }

          start_date = new Date(start_date);
          var start_year = start_date.getFullYear();
          start_month = start_date.getMonth();

          index = 0;
          var modulu = 0;
          var flag;

          for (i = 0; i < eventsArray.length; i++) {
            flag = true;
            while (flag) {
              index = modulu % recurenceObject.when.length;
              if (modulu !== 0 && index == 0) {
                start_date = moment(start_date).add(1, 'months');
                start_date = new Date(start_date);
                start_year = start_date.getFullYear();
                start_month = start_date.getMonth();
              }
              if (dateIsValid(arrayOfDays[index], start_month, start_year)) {
                start = moment([start_year, start_month, arrayOfDays[index]]);
                start = new Date(start);
                start_date = new Date(start_date);
                if (start.getTime() < start_date.getTime()) {
                  start = getMonthlyEventStartDate(arrayOfDays[index], start_date)
                }

                eventsArray[i].startDate = getNewDateAndTime(start_date, start);
                eventsArray[i].eventDuration = recurenceObject.eventDuration;
                eventsArray[i].durationUnits = recurenceObject.durationUnits;
                modulu++;
                flag = false;
              } else {
                modulu++;
              }
            }
          }

          ViewstartDate = moment(eventsArray[0].startDate);
          ViewendDate = moment(eventsArray[eventsArray.length - 1].startDate);

          for (var i = 0; i < eventsArray.length; i++) {
            eventsArray[i].endDate = dateAdd(eventsArray[i].startDate, eventsArray[i].eventDuration, eventsArray[i].durationUnits);
            eventsArray[i].initialStartDate = eventsArray[i].startDate;
            eventsArray[i].initialEventDuration = eventsArray[i].eventDuration;
            eventsArray[i].initialDurationUnits = eventsArray[i].durationUnits;
            eventsArray[i].isDirty = false;
          }

          window.eventsArray = eventsArray;
          window.StartingDate = ViewstartDate;
          window.endingDate = ViewendDate;
          window.flag = true;
          window.zoomed = false;

          updateTimelineDuration(ViewstartDate, ViewendDate);
          //renderEvents(eventsArray, ViewstartDate, ViewendDate);
        }                                                         // End of - "create X events" of range_of_recurence was selected (for !isNaN)

        if (recurenceObject.nrOfEvents == "") {                  // A "Repeat until" or "no_end_date" has bin selected (for !isNaN)

          start_date = getStartDateForSelectedWeekday(recurenceObject);
          start_date = new Date(start_date);
          start_month = start_date.getMonth();
          selectedDayNumber = moment(start_date).day();
          index = 0;

          arrayOfDays = [];

          for (i = 0; i < recurenceObject.when.length; i++) {      // find the start day dependent on the startDate (selected in the menu)
            if (moment(recurenceObject.startDate).date() <= recurenceObject.when[i]) {
              numberOfDays = recurenceObject.when[i] - moment(recurenceObject.startDate).date();
              start_date = moment(recurenceObject.startDate).add(numberOfDays, 'days');
              index = i;                                      // index of the found next day...
              found = true;
              break;
            }
          }

          if (!found) {
            numberOfDaysInMonth = moment(recurenceObject.startDate).daysInMonth();
            numberOfDays = (numberOfDaysInMonth - recurenceObject.when[1]) + 1;
            start_date = moment(recurenceObject.startDate).add(numberOfDays, 'days');
            for (i = 0; i < recurenceObject.when.length; i++) {
              arrayOfDays.push({});
              arrayOfDays[i] = recurenceObject.when[i];
            }
          }

          if (found) {
            for (i = 0; i < recurenceObject.when.length; i++) {
              arrayOfDays.push({});
              arrayOfDays[i] = recurenceObject.when[i];
            }

            partOfArray = arrayOfDays.slice(index);
            for (j = 0; j < partOfArray.length; j++) {
              arrayOfDays.pop();
            }
            arrayOfDays = partOfArray.concat(arrayOfDays);
          }

          start_date = new Date(start_date);
          var start_year = start_date.getFullYear();
          start_month = start_date.getMonth();

          index = 0;
          var start = 0;
          var eventArrayIndex = 0;

          ViewendDate = moment(recurenceObject.endDate);
          begin_date = new Date(recurenceObject.startDate);

          while (moment(start_date) < moment(ViewendDate)) {
            var x = index % recurenceObject.when.length;
            if (x == 0 && index == 0) {
              start = new Date(begin_date);
              start_year = start.getFullYear();
              start_month = start.getMonth();
              start = moment([start_year, start_month, recurenceObject.when[x]]);
              if (moment(start) > moment(ViewendDate)) {
                break;
              }
              if (start >= begin_date) {
                start_date = moment(start);
                if (dateIsValid(recurenceObject.when[x], start_month, start_year)) {
                  eventsArray.push({});
                  eventsArray[eventArrayIndex].startDate = moment(start).format();
                  eventsArray[eventArrayIndex].eventDuration = recurenceObject.eventDuration;
                  eventsArray[eventArrayIndex].durationUnits = recurenceObject.durationUnits;
                  start_date = moment(start);
                  eventArrayIndex++;
                }
              } else {
                start_date = moment(begin_date);
              }
            }

            if (x == 0 && index !== 0) {
              start = moment(start_date).add(1, 'months');
              start = new Date(start);
              start_year = start.getFullYear();
              start_month = start.getMonth();
              start = moment([start_year, start_month, recurenceObject.when[x]]);
              start_date = moment(start);
              if (moment(start) > moment(ViewendDate)) {
                break;
              }
              if (start >= begin_date) {
                if (dateIsValid(recurenceObject.when[x], start_month, start_year)) {
                  eventsArray.push({});
                  eventsArray[eventArrayIndex].startDate = moment(start).format();
                  eventsArray[eventArrayIndex].eventDuration = recurenceObject.eventDuration;
                  eventsArray[eventArrayIndex].durationUnits = recurenceObject.durationUnits;
                  start_date = moment(start);
                  eventArrayIndex++;
                }
              } else {
                start_date = moment(start);
              }
            } else {
              if (x !== 0) {
                start = moment([start_year, start_month, recurenceObject.when[x]]);
                if (moment(start) > moment(ViewendDate)) {
                  break;
                }
                if (start >= begin_date) {
                  if (moment(start) > moment(ViewendDate)) {
                    break;
                  }
                  if (dateIsValid(recurenceObject.when[x], start_month, start_year)) {
                    eventsArray.push({});
                    eventsArray[eventArrayIndex].startDate = moment(start).format();
                    eventsArray[eventArrayIndex].eventDuration = recurenceObject.eventDuration;
                    eventsArray[eventArrayIndex].durationUnits = recurenceObject.durationUnits;
                    start_date = moment(start);
                    eventArrayIndex++;
                  }
                  start_date = moment(start);
                }
              }
            }
            index++;
          }

          if (eventsArray.length > 0) {
            ViewstartDate = moment(eventsArray[0].startDate);
            ViewendDate = moment(eventsArray[eventsArray.length - 1].startDate);

            for (var i = 0; i < eventsArray.length; i++) {
              eventsArray[i].endDate = dateAdd(eventsArray[i].startDate, eventsArray[i].eventDuration, eventsArray[i].durationUnits);
              eventsArray[i].initialStartDate = eventsArray[i].startDate;
              eventsArray[i].initialEventDuration = eventsArray[i].eventDuration;
              eventsArray[i].initialDurationUnits = eventsArray[i].durationUnits;
              eventsArray[i].isDirty = false;
            }

            window.eventsArray = eventsArray;
            window.StartingDate = ViewstartDate;
            window.endingDate = ViewendDate;
            window.flag = true;
            window.zoomed = false;

            updateTimelineDuration(ViewstartDate, ViewendDate);
            //renderEvents(eventsArray, ViewstartDate, ViewendDate);

          } else {
            untilDate = $('.range_of_recurence input#repeat_until.form-control.date-field.datepick');
            untilDate[0].value = "";
            $(".range_of_recurence").addClass("input_warning");
          }
        }                                                         // A "Repeat until" or "no_end_date" has bin selected (for !isNaN)
      }                                                           // End of - "recurence_pattern" seelcted is e.g. 01, 02, 29,...,31

  }                                                               // End of - SWITCH FOR DAILY, WEEKLY & MONTHLY
}                                                                 // End of - Function to prepare events for rendering
//__________________________________________________________________________________
// Function to get the weekly start date for an event
//==================================================================================
function getWeeklyEventStartDate(selectedWeekDayNumber, startWeekDayNumber, startDate) {

  var nrOfDays;
  var start_date;

  selectedWeekDayNumber = moment().day(selectedWeekDayNumber);
  selectedWeekDayNumber = moment(selectedWeekDayNumber).day();                    // get the weekday number of a selected weekday (e.g. Monday, Thursday,...)
  start_date = moment(startDate);                                         // calculate the start_date for the weekday to start events
  nrOfDays = selectedWeekDayNumber - startWeekDayNumber;                  // get the difference in days between the input startDate and the new weekday
  if (nrOfDays == 0) {
    start_date = moment(startDate);                                       // if '0' then it is the same day as the input startDate
  } else {
    if (nrOfDays > 0) {
      start_date = moment(start_date).add(nrOfDays, 'days');              // if event day is ahead from startDate, then add days to the input startDate
    } else {
      nrOfDays = (7 - startWeekDayNumber) + selectedWeekDayNumber;        // if input startDate is ahead
      start_date = moment(start_date).add(nrOfDays, 'days');              // add days to the selected weekday
    }
  }
  return start_date.format();
}

//__________________________________________________________________________________
// Function to get the monthly start date for an event
//==================================================================================
function getMonthlyEventStartDate(selectedMonthDay, startDate) {
  startDate = new Date(startDate);

  var start_year = startDate.getFullYear();
  var start_month = startDate.getMonth();
  var day_of_month = startDate.getDate();
  if (day_of_month > selectedMonthDay) {
    var start_day = selectedMonthDay;
    var start_date = moment([start_year, start_month, start_day]);
    start_date = start_date.add(1, 'months');
  } else {
    start_day = selectedMonthDay;
    start_date = moment([start_year, start_month, start_day]);
    start_day = selectedMonthDay;
    start_date = moment([start_year, start_month, start_day]);
  }
  return start_date.format();
}

//__________________________________________________________________________________
// Count the times a selected week day appearce between start and end date
//==================================================================================
function countSelectedWeekdayAppearance(startdate, enddate, selectedDayOfWeekNumber) {   // How many times a selected weekday (e.g. Monday) appears
  var endDate = moment(enddate);
  var startDate = moment(startdate);                                  // The start date is when the selected weekday appears for the first time
  var difDays = endDate.diff(startDate, 'days') + 2;                      // How many says between start and end date
  var nextDay = moment(startDate);
  var numberOfWeekDayInMonth = 0;

  //for (var i = 0; i <= difDays; i++) {
  for (var i = 0; i < difDays; i++) {                                  // Loop to find the number of times the selected weekday appears
    if (nextDay.day() === selectedDayOfWeekNumber)                    // Selected weekday found ?
    {
      numberOfWeekDayInMonth++;                                   // update counter
    }
    nextDay = nextDay.add(1, 'days');                              // move to next day. Needed to know if the day is the one searched
  }
  return numberOfWeekDayInMonth;                                    // return counter
}

//__________________________________________________________________________________
// Get the start date for a selected week day
//==================================================================================
function getStartDateForSelectedWeekday(recurenceObject) {           // calculate the start date for the selected weekday
  var start_date = moment(recurenceObject.startDate);
  var firstDayOfMonthNumber = start_date.day();
  var selectedDayOfWeekNumber = moment().day(recurenceObject.when[1]);
  selectedDayOfWeekNumber = moment(selectedDayOfWeekNumber).day();

  var order = getOrder(recurenceObject);


  if (selectedDayOfWeekNumber == 0) {           // Sunday
    var addDays = 7 - firstDayOfMonthNumber;
    var new_date = moment(start_date).add(addDays, 'days');
    start_date = new Date(start_date);
    new_date = new Date(new_date);
    if (new_date.getTime() > start_date.getTime()) {
      if (order == 1) {
        new_date = getTheNextEventDate(moment(new_date), order, recurenceObject);
      }

    }

    if (order > 1) {
      var weekNumber = Math.ceil(moment(new_date).date() / 7);
      if (order > weekNumber) {
        addDays = order - weekNumber;
        new_date = moment(new_date).add(addDays * 7, 'days');
      }

      if (order < weekNumber) {
        new_date = getTheNextEventDate(moment(new_date), order, recurenceObject);
      }
    }
  }

  if (firstDayOfMonthNumber == selectedDayOfWeekNumber) {
    new_date = moment(start_date);
    weekNumber = Math.ceil(moment(new_date).date() / 7);
    if (order > weekNumber) {
      addDays = order - weekNumber;
      new_date = moment(new_date).add(addDays * 7, 'days');
    }

    if (order < weekNumber) {
      new_date = getTheNextEventDate(moment(new_date), order, recurenceObject);
    }

    if (order == 0) {
      return moment(start_date).format();
    }
  }

  if (firstDayOfMonthNumber > selectedDayOfWeekNumber && selectedDayOfWeekNumber !== 0) {
    var minusDays = firstDayOfMonthNumber - selectedDayOfWeekNumber;
    new_date = moment(start_date).subtract(minusDays, 'days');
    if (order == 0) {
      new_date = moment(new_date).add(7, 'days');
    }

    if (order == 1) {
      new_date = getTheNextEventDate(moment(new_date), order, recurenceObject);
    }

    if (order > 1) {
      weekNumber = Math.ceil(moment(new_date).date() / 7);
      if (order > weekNumber) {
        addDays = order - weekNumber;
        new_date = moment(new_date).add(addDays * 7, 'days');
      }

      if (order < weekNumber) {
        new_date = getTheNextEventDate(moment(new_date), order, recurenceObject);
      }

      if (order == weekNumber) {
        new_date = getTheNextEventDate(moment(new_date), order, recurenceObject);
      }
    }
  }

  if (firstDayOfMonthNumber < selectedDayOfWeekNumber) {
    addDays = selectedDayOfWeekNumber - firstDayOfMonthNumber;
    new_date = moment(start_date).add(addDays, 'days');

    weekNumber = Math.ceil(moment(new_date).date() / 7);
    if (order > weekNumber) {
      addDays = order - weekNumber;
      new_date = moment(new_date).add(addDays * 7, 'days');
    }

    if (weekNumber > order) {
      if (order > 0) {
        new_date = getTheNextEventDate(moment(new_date), order, recurenceObject);
      }
    }

  }

  return moment(new_date).format();
}

//__________________________________________________________________________________
// Function to render events
//==================================================================================
function renderEvents(eventsArray, ViewstartDate, ViewendDate) {     //Function is triggered by prepareEvents

  $('body').css({ 'cursor': 'wait' });

  // Remove existing 'highlighter' bars, before re-rendering
  if ($('.highlighter').length != 0) {
    $('.highlighter').remove();
  }

  tzOffset();

  var isRTL = (fullcalendarScheduler.getOption('dir') == 'rtl');
  var view = fullcalendarScheduler.view;

  var viewStartDate = new Date(view.currentStart);
  var viewEndDate = new Date(view.currentEnd);

  var viewStartTime = viewStartDate.getTime();
  var viewEndTime = viewEndDate.getTime();
  var viewDuration = viewEndTime - viewStartTime;


  for (var i = 0; i < eventsArray.length; i++) {
    var duration = eventsArray[i].eventDuration;
    var durationUnits = eventsArray[i].durationUnits;

    $('#fullcalendar_scheduler tbody.fc-body td.fc-time-area div.fc-scroller div.fc-scroller-canvas').prepend($('<div id="highlighter-' + i + '" class="highlighter" draggable="true"></div>'));

    var startDate = new Date(eventsArray[i].startDate)
    var width;

    view = fullcalendarScheduler.view;
    viewStartDate = new Date(view.currentStart);
    viewEndDate = new Date(view.currentEnd);
    viewStartTime = viewStartDate.getTime();
    viewEndTime = viewEndDate.getTime();
    viewDuration = viewEndTime - viewStartTime;
    width = setHighlighterBarWidth(duration, durationUnits)

    $('#highlighter-' + i).css('width', width);

    if (isRTL) {
      $('#highlighter-' + i).css('left', getEventWindowWidth() - ((((startDate.getTime() + tzOffset()) - viewStartTime) / viewDuration) * getEventWindowWidth()) - width);
    } else {
      $('#highlighter-' + i).css('left', (((startDate.getTime() - viewStartTime) / viewDuration) * getEventWindowWidth()));
      //$('#highlighter-' + i).css('left', (((startDate.getTime() + tzOffset()) - viewStartTime) / viewDuration) * getEventWindowWidth());
    }

  }

  $('.highlighter').height(getEventWindowHeight());
  $('.highlighter').draggable({ axis: "x" });
  $('.highlighter').resizable({ handles: "e, w" });
  $('.highlighter').css('visibility', 'visible');

  $('.highlighter').on('drag', function (event, ui) {
    onHighlighterBarUpdate(event, ui, $(this));
    renderSelectedResources();
    showTooltip(event, ui, $(this));
  });

  $('.highlighter').on('resize', function (event, ui) {
    onHighlighterBarUpdate(event, ui, $(this));
    renderSelectedResources();
    showTooltip(event, ui, $(this));
  });

  $('.highlighter').on('mouseup', function (event, ui) {
    // after drag/resize operation
  });

  $('.highlighter').on('mouseenter', function (event, ui) {
    showTooltip(event, ui, $(this));
  });

  $('.highlighter').on('mouseleave', function (event, ui) {
    hideTooltip(event, ui, $(this));
  });

  $('.highlighter').on('dblclick', function (event, ui) {
    dblclickEventZoomIn(event, ui, $(this));
  });

  $('body').css({ 'cursor': 'auto' });

}

function clearHighlighterConflicts() {
  if (typeof eventsArray !== 'undefined') {
    for (var eventArrayIndex = 0; eventArrayIndex < eventsArray.length; eventArrayIndex++) {
      eventsArray[eventArrayIndex].isInConflict = false;
    }
  }
}

function displayHighlighterConflicts() {
  if (typeof eventsArray !== 'undefined') {
    for (var eventArrayIndex = 0; eventArrayIndex < eventsArray.length; eventArrayIndex++) {
      // TODO: These values should come from the database
      var highlighter_color = (eventsArray[eventArrayIndex].isInConflict == true) ? 'rgba(255, 0, 0, 0.5)' : 'rgba(0, 255, 0, 0.5)';
      $('#highlighter-' + eventArrayIndex).css('background-color', highlighter_color);
    }
  }
}

//__________________________________________________________________________________
// get start date for a daily event
//==================================================================================
function getDailyEventStartDate(start_date, howOften) {

  start_date = moment(start_date).add(howOften, 'days');
  start_date = moment(start_date).format();
  return start_date;
}

//__________________________________________________________________________________
// get order of selected day, should it appear every week (zero for 'every',...) or the first day in a month ('1' for 'first',...)
//==================================================================================
function getOrder(recurenceObject) {
  var whichOrder = ["every", "first", "second", "third", "fourth", "fifth", "last"];
  var order = whichOrder.indexOf(recurenceObject.when[0]);
  return order;
}

//__________________________________________________________________________________
// Get the next date the event should appear
//==================================================================================
function getTheNextEventDate(previousEventDate, order, recurenceObject) {

  previousEventDate = new Date(previousEventDate);
  var start_date;

  var daysInMonth = moment(previousEventDate).daysInMonth();
  var difDays = daysInMonth - moment(previousEventDate).date();
  var modulu = difDays % 7;
  modulu = 7 - modulu;

  if (modulu == 0) {
    if (order == 0) {
      start_date = moment(previousEventDate).add(difDays + 7, 'days');
    }

    if (order == 1) {
      start_date = moment(previousEventDate).add(difDays + 7, 'days');
      if (recurenceObject.howOften > 1) {
        start_date = getDateAfterNumberOfMonths(start_date, (recurenceObject.howOften - 1), order);
      }
    }

    if (order > 1) {
      start_date = moment(previousEventDate).add(difDays + 7 + ((order - 1) * 7), 'days');
      if (recurenceObject.howOften > 1) {
        start_date = getDateAfterNumberOfMonths(start_date, (recurenceObject.howOften - 1), order);
      }
    }
  }

  if (modulu > 0) {
    if (order == 0) {
      start_date = moment(previousEventDate).add(difDays + modulu, 'days');
      if (recurenceObject.howOften > 1) {
        start_date = getDateAfterNumberOfMonths(start_date, (recurenceObject.howOften - 1), order);
      }
    }

    if (order == 1) {
      start_date = moment(previousEventDate).add(difDays + modulu, 'days');
      if (recurenceObject.howOften > 1) {
        start_date = getDateAfterNumberOfMonths(start_date, (recurenceObject.howOften - 1), order);
      }
    }

    if (order > 1) {
      start_date = moment(previousEventDate).add(difDays + modulu + ((order - 1) * 7), 'days');
      if (recurenceObject.howOften > 1) {
        start_date = getDateAfterNumberOfMonths(start_date, (recurenceObject.howOften - 1), order);
      }
    }
  }

  return moment(start_date).format();
}

//__________________________________________________________________________________
// Get the difference in days between two dates
//==================================================================================
function getTwoDatesDifference(date1, date2) {
  date1 = new Date(date1);
  date2 = new Date(date2);
  var timeDiff = Math.abs(date2.getTime() - date1.getTime());
  var diffHours = Math.ceil(timeDiff / (1000 * 3600));
  return diffHours;
}

//__________________________________________________________________________________
// Find the next date for the event after 1 month, 2, 3,...
//==================================================================================
function getDateAfterNumberOfMonths(currentDate, numberOfMonths, order) {

  if (numberOfMonths == 0) {
    weekNumber = Math.ceil(moment(currentDate).date() / 7);
  }

  new_date = moment(currentDate);

  for (var i = 0; i < numberOfMonths; i++) {
    var daysInMonth = moment(new_date).daysInMonth();
    var difDays = daysInMonth - moment(new_date).date();
    var modulu = difDays % 7;
    modulu = 7 - modulu;
    var addDays = difDays + modulu;
    var new_date = moment(new_date).add(addDays, 'days');
    var weekNumber = Math.ceil(moment(new_date).date() / 7);
    if (order > weekNumber) {
      addDays = order - weekNumber;
      new_date = moment(new_date).add(addDays * 7, 'days');
    }
  }

  return new_date;
}

//__________________________________________________________________________________
// Get number of days to the next event date
//==================================================================================
function getNumberOfDaysToNewDate(currentDate, numberOfMonths, order) {

  for (var i = 0; i < numberOfMonths; i++) {
    var daysInMonth = moment(currentDate).daysInMonth();
    var difDays = daysInMonth - moment(currentDate).date();
    var modulu = difDays % 7;
    modulu = 7 - modulu;
    var addDays = difDays + modulu + ((order - 1) * 7);
    var new_date = moment(currentDate).add(addDays, 'days');
  }

  return new_date;
}

//__________________________________________________________________________________
// Sort function
//==================================================================================
function sortArray(a, b) {
  const startDateA = a.startDate.toUpperCase();
  const startDateB = b.startDate.toUpperCase();

  let comparison = 0;
  if (startDateA > startDateB) {
    comparison = 1;
  } else if (startDateA < startDateB) {
    comparison = -1;
  }
  return comparison;
}

//__________________________________________________________________________________
// Set the time in the selected date
//==================================================================================
function getNewDateAndTime(start_date, start) {
  start_date = new Date(start_date);
  var hours = start_date.getHours();
  var minutes = start_date.getMinutes();
  start = new Date(start);
  start = start.setHours(hours);
  start = new Date(start);
  start = start.setMinutes(minutes);
  return moment(start).format();
}

function dateIsValid(selectedDay, month, year) {
  var date = moment([year, month, 1]);
  var daysInMonth = moment(date).daysInMonth();

  if (selectedDay > daysInMonth) {
    return false;
  } else {
    return true;
  }
}

//==================================================================
function getDurationInUnits(eventDurationHours, durationUnits) {

  var duration;

  switch (durationUnits) {
    case 'day':
      duration = (eventDurationHours / 24).toFixed(2);
      break;
    case 'hour':
      duration = (eventDurationHours).toFixed(2);
      break;
    case 'minute':
      duration = Math.round(eventDurationHours * 60);
      break;
    default:
      duration = Math.round(eventDurationHours * 60);
  }
  return duration;
}                                                        // End of getDurationInUnits