//
// FullcalendarTimeline - Resource manipulation controls
//

// Initialise timeline manipulation controls
FullcalendarTimeline.prototype.initialiseResourceManipulationControls = function () {
  this._selectedResources = [];

  // TODO: selectedBackgroundColour values should come from the application or database configuration
  //  values are ordered: label (resource), body (timeline)
  this._unselectedBackgroundColour = ['rgba(0, 0, 0, 0)', 'rgba(0, 0, 0, 0)'];
  //this._selectedBackgroundColour = ['LightSeaGreen', 'DarkSeaGreen'];
  this._selectedBackgroundColour = ['LightSeaGreen', 'rgba(0, 0, 0, 0)'];
  this._conflictBackgroundColour = ['IndianRed', 'LightSalmon'];

  // for resourceSelectedHandler() initialisation, see resourceRender callback in fullcalendar initialisation
  this.renderSelectedResources();
}

FullcalendarTimeline.prototype.resourceSelectedHandler = function (renderInfo) {
  var resourceId = renderInfo.resource.id

  var index = this._selectedResources.map(function (e) { return e.resourceId; }).indexOf(resourceId);

  if (index == -1) {
    // Add resource to _selectedResources
    var resourceData = {};
    resourceData.resourceId = resourceId;
    resourceData.isInConflict = false;
    resourceData.conflictingEvents = [];
    resourceData.conflictingRestrictions = [];

    this._selectedResources.push(resourceData);
  } else {
    // Remove resource from _selectedResources
    this._selectedResources = this._selectedResources.filter(function (value, index, arr) {
      return value.resourceId != resourceId;
    });
  }

  this.renderSelectedResources();

  // labelTds.css('background', 'blue');
  // bodyTds.css('background', 'yellow');
  // resource_id = labelTds.parent().attr("data-resource-id");
  // labelTds.eq(0).find('.fc-cell-content')
  //   .append(
  //     $('<strong>('+resource_id+')</strong>')
  //     //$('<strong>(?)</strong>')
  //     //.popover({
  //     //  title: resourceObj.title,
  //     //  content: 'test!',
  //     //  trigger: 'hover',
  //     //  placement: 'bottom',
  //     //  container: 'body'
  //     //})
  //   );
}

FullcalendarTimeline.prototype.renderSelectedResources = function () {
  // Clear all highlighting on resources
  this._timelineViewWrapper.find('.fc-resource-area [data-resource-id]').css('background-color', this._unselectedBackgroundColour[0]);
  this._timelineViewWrapper.find('.fc-time-area [data-resource-id]').css('background-color', this._unselectedBackgroundColour[1]);

  // Check for conflicts
  this.flagResourceConflicts();

  // Render event conflicts
  this.displayHighlighterConflicts();

  // Render resource conflicts
  for (var i = 0; i < this._selectedResources.length; i++) {
    var resourceAreaColour = (this._selectedResources[i].isInConflict) ? this._conflictBackgroundColour[0] : this._selectedBackgroundColour[0];
    var timeAreaColour = (this._selectedResources[i].isInConflict) ? this._conflictBackgroundColour[1] : this._selectedBackgroundColour[1];

    this._timelineViewWrapper.find('.fc-resource-area [data-resource-id="' + this._selectedResources[i].resourceId + '"]').css('background-color', resourceAreaColour);

    this._timelineViewWrapper.find('.fc-time-area [data-resource-id="' + this._selectedResources[i].resourceId + '"]').css('background-color', timeAreaColour);
  }
}

FullcalendarTimeline.prototype.flagResourceConflicts = function () {
  // Mechanism to pass (this) to the handlers
  var fullCalendarTimelineObject = this;

  $.when(
    // Get existing events and resource restrictions
    $.ajax({ url: '/resource_planner/resource_events.json', async: false }),
    $.ajax({ url: '/resource_planner/resource_restrictions.json', async: false })
  ).done(function (a1, a2) {
    var existingEvents = a1[0];
    var resourceRestrictions = a2[0];

    // get currently planned event list (eventArray)
    var eventsArray = fullCalendarTimelineObject._eventsArray

    // see where they conflict and flag the resource
    if (typeof eventsArray !== 'undefined') {

      // Clear active conflicts on the highlighter bar
      fullCalendarTimelineObject.clearHighlighterConflicts();

      for (var selectedResourceindex = 0; selectedResourceindex < fullCalendarTimelineObject._selectedResources.length; selectedResourceindex++) {

        fullCalendarTimelineObject._selectedResources[selectedResourceindex].isInConflict = false;
        fullCalendarTimelineObject._selectedResources[selectedResourceindex].conflictingEvents = [];
        fullCalendarTimelineObject._selectedResources[selectedResourceindex].conflictingRestrictions = [];

        for (var eventArrayIndex = 0; eventArrayIndex < eventsArray.length; eventArrayIndex++) {

          for (var i = 0; i < existingEvents.length; i++) {
            if (existingEvents[i].resourceIds.indexOf(fullCalendarTimelineObject._selectedResources[selectedResourceindex].resourceId) != -1) {

              var resourceQuantity = parseInt(fullCalendarTimelineObject._selectedResources[selectedResourceindex].resourceId.split("-")[2], 10);

              // TODO: Arbitrate for resourceQuantity > 1
              if (resourceQuantity > 0 &&
                (dateCheck(existingEvents[i].start, existingEvents[i].end, eventsArray[eventArrayIndex].startDate)
                  || dateCheck(existingEvents[i].start, existingEvents[i].end, eventsArray[eventArrayIndex].endDate)
                )
              ) {
                fullCalendarTimelineObject._selectedResources[selectedResourceindex].isInConflict = true;
                var conflictingEvent = {};
                conflictingEvent.id = existingEvents[i].id;
                conflictingEvent.name = existingEvents[i].title;
                fullCalendarTimelineObject._selectedResources[selectedResourceindex].conflictingEvents.push(conflictingEvent);
                eventsArray[eventArrayIndex].isInConflict = true;
              }
            }
          }

          for (var j = 0; j < resourceRestrictions.length; j++) {
            if (resourceRestrictions[j].resourceIds.indexOf(fullCalendarTimelineObject._selectedResources[selectedResourceindex].resourceId) != -1) {
              if (dateCheck(resourceRestrictions[j].start, resourceRestrictions[j].end, eventsArray[eventArrayIndex].startDate)
                || dateCheck(resourceRestrictions[j].start, resourceRestrictions[j].end, eventsArray[eventArrayIndex].endDate)
              ) {
                fullCalendarTimelineObject._selectedResources[selectedResourceindex].isInConflict = true;
                var conflictingRestriction = {};
                conflictingRestriction.id = existingEvents[j].id;
                conflictingRestriction.name = existingEvents[j].title;
                fullCalendarTimelineObject._selectedResources[selectedResourceindex].conflictingRestrictions.push(conflictingRestriction);
                eventsArray[eventArrayIndex].isInConflict = true;
              }
            }
          }

        }
      }

      // Update the 'global' eventsArray
      fullCalendarTimelineObject._eventsArray = eventsArray;
    }
  });
}
